import type { BaseContract, BigNumber, BigNumberish, BytesLike, CallOverrides, ContractTransaction, Overrides, PopulatedTransaction, Signer, utils } from "ethers";
import type { FunctionFragment, Result, EventFragment } from "@ethersproject/abi";
import type { Listener, Provider } from "@ethersproject/providers";
import type { TypedEventFilter, TypedEvent, TypedListener, OnEvent, PromiseOrValue } from "./common";
export interface SAFEDeployerInterface extends utils.Interface {
    functions: {
        "deploySafe(address,uint256)": FunctionFragment;
        "deploySafeProxy(uint256)": FunctionFragment;
        "fallbackModule()": FunctionFragment;
        "modules(uint256)": FunctionFragment;
        "modulesLib()": FunctionFragment;
        "owner()": FunctionFragment;
        "ownersList(uint256)": FunctionFragment;
        "renounceOwnership()": FunctionFragment;
        "safeProxyFactory()": FunctionFragment;
        "safeSingleton()": FunctionFragment;
        "transferOwnership(address)": FunctionFragment;
        "updateFallbackModule(address)": FunctionFragment;
        "updateModulesLib(address)": FunctionFragment;
        "updateModulesList(address[])": FunctionFragment;
        "updateSafeProxyFactory(address)": FunctionFragment;
        "updateSafeSingleton(address)": FunctionFragment;
        "userAmounts(address)": FunctionFragment;
        "users(uint256)": FunctionFragment;
    };
    getFunction(nameOrSignatureOrTopic: "deploySafe" | "deploySafeProxy" | "fallbackModule" | "modules" | "modulesLib" | "owner" | "ownersList" | "renounceOwnership" | "safeProxyFactory" | "safeSingleton" | "transferOwnership" | "updateFallbackModule" | "updateModulesLib" | "updateModulesList" | "updateSafeProxyFactory" | "updateSafeSingleton" | "userAmounts" | "users"): FunctionFragment;
    encodeFunctionData(functionFragment: "deploySafe", values: [PromiseOrValue<string>, PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "deploySafeProxy", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "fallbackModule", values?: undefined): string;
    encodeFunctionData(functionFragment: "modules", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "modulesLib", values?: undefined): string;
    encodeFunctionData(functionFragment: "owner", values?: undefined): string;
    encodeFunctionData(functionFragment: "ownersList", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "renounceOwnership", values?: undefined): string;
    encodeFunctionData(functionFragment: "safeProxyFactory", values?: undefined): string;
    encodeFunctionData(functionFragment: "safeSingleton", values?: undefined): string;
    encodeFunctionData(functionFragment: "transferOwnership", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "updateFallbackModule", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "updateModulesLib", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "updateModulesList", values: [PromiseOrValue<string>[]]): string;
    encodeFunctionData(functionFragment: "updateSafeProxyFactory", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "updateSafeSingleton", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "userAmounts", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "users", values: [PromiseOrValue<BigNumberish>]): string;
    decodeFunctionResult(functionFragment: "deploySafe", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "deploySafeProxy", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "fallbackModule", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "modules", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "modulesLib", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "owner", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "ownersList", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "renounceOwnership", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "safeProxyFactory", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "safeSingleton", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "transferOwnership", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "updateFallbackModule", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "updateModulesLib", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "updateModulesList", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "updateSafeProxyFactory", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "updateSafeSingleton", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "userAmounts", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "users", data: BytesLike): Result;
    events: {
        "Initialiser(bytes)": EventFragment;
        "OwnershipTransferred(address,address)": EventFragment;
        "SafeProxyDeploy(address,uint256)": EventFragment;
        "SafeProxyDeployed(address,address)": EventFragment;
    };
    getEvent(nameOrSignatureOrTopic: "Initialiser"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "OwnershipTransferred"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "SafeProxyDeploy"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "SafeProxyDeployed"): EventFragment;
}
export interface InitialiserEventObject {
    data: string;
}
export type InitialiserEvent = TypedEvent<[string], InitialiserEventObject>;
export type InitialiserEventFilter = TypedEventFilter<InitialiserEvent>;
export interface OwnershipTransferredEventObject {
    previousOwner: string;
    newOwner: string;
}
export type OwnershipTransferredEvent = TypedEvent<[
    string,
    string
], OwnershipTransferredEventObject>;
export type OwnershipTransferredEventFilter = TypedEventFilter<OwnershipTransferredEvent>;
export interface SafeProxyDeployEventObject {
    user: string;
    counter: BigNumber;
}
export type SafeProxyDeployEvent = TypedEvent<[
    string,
    BigNumber
], SafeProxyDeployEventObject>;
export type SafeProxyDeployEventFilter = TypedEventFilter<SafeProxyDeployEvent>;
export interface SafeProxyDeployedEventObject {
    user: string;
    proxy: string;
}
export type SafeProxyDeployedEvent = TypedEvent<[
    string,
    string
], SafeProxyDeployedEventObject>;
export type SafeProxyDeployedEventFilter = TypedEventFilter<SafeProxyDeployedEvent>;
export interface SAFEDeployer extends BaseContract {
    connect(signerOrProvider: Signer | Provider | string): this;
    attach(addressOrName: string): this;
    deployed(): Promise<this>;
    interface: SAFEDeployerInterface;
    queryFilter<TEvent extends TypedEvent>(event: TypedEventFilter<TEvent>, fromBlockOrBlockhash?: string | number | undefined, toBlock?: string | number | undefined): Promise<Array<TEvent>>;
    listeners<TEvent extends TypedEvent>(eventFilter?: TypedEventFilter<TEvent>): Array<TypedListener<TEvent>>;
    listeners(eventName?: string): Array<Listener>;
    removeAllListeners<TEvent extends TypedEvent>(eventFilter: TypedEventFilter<TEvent>): this;
    removeAllListeners(eventName?: string): this;
    off: OnEvent<this>;
    on: OnEvent<this>;
    once: OnEvent<this>;
    removeListener: OnEvent<this>;
    functions: {
        deploySafe(owner: PromiseOrValue<string>, nonce: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        deploySafeProxy(counter: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        fallbackModule(overrides?: CallOverrides): Promise<[string]>;
        modules(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        modulesLib(overrides?: CallOverrides): Promise<[string]>;
        owner(overrides?: CallOverrides): Promise<[string]>;
        ownersList(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        renounceOwnership(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        safeProxyFactory(overrides?: CallOverrides): Promise<[string]>;
        safeSingleton(overrides?: CallOverrides): Promise<[string]>;
        transferOwnership(newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        updateFallbackModule(_fallbackModule: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        updateModulesLib(_modulesLib: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        updateModulesList(_modulesList: PromiseOrValue<string>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        updateSafeProxyFactory(_safeProxyFactory: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        updateSafeSingleton(_safeSingleton: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        userAmounts(arg0: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[BigNumber]>;
        users(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
    };
    deploySafe(owner: PromiseOrValue<string>, nonce: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    deploySafeProxy(counter: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    fallbackModule(overrides?: CallOverrides): Promise<string>;
    modules(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    modulesLib(overrides?: CallOverrides): Promise<string>;
    owner(overrides?: CallOverrides): Promise<string>;
    ownersList(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    renounceOwnership(overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    safeProxyFactory(overrides?: CallOverrides): Promise<string>;
    safeSingleton(overrides?: CallOverrides): Promise<string>;
    transferOwnership(newOwner: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    updateFallbackModule(_fallbackModule: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    updateModulesLib(_modulesLib: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    updateModulesList(_modulesList: PromiseOrValue<string>[], overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    updateSafeProxyFactory(_safeProxyFactory: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    updateSafeSingleton(_safeSingleton: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    userAmounts(arg0: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
    users(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    callStatic: {
        deploySafe(owner: PromiseOrValue<string>, nonce: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        deploySafeProxy(counter: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        fallbackModule(overrides?: CallOverrides): Promise<string>;
        modules(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        modulesLib(overrides?: CallOverrides): Promise<string>;
        owner(overrides?: CallOverrides): Promise<string>;
        ownersList(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        renounceOwnership(overrides?: CallOverrides): Promise<void>;
        safeProxyFactory(overrides?: CallOverrides): Promise<string>;
        safeSingleton(overrides?: CallOverrides): Promise<string>;
        transferOwnership(newOwner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        updateFallbackModule(_fallbackModule: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        updateModulesLib(_modulesLib: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        updateModulesList(_modulesList: PromiseOrValue<string>[], overrides?: CallOverrides): Promise<void>;
        updateSafeProxyFactory(_safeProxyFactory: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        updateSafeSingleton(_safeSingleton: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        userAmounts(arg0: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        users(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    };
    filters: {
        "Initialiser(bytes)"(data?: null): InitialiserEventFilter;
        Initialiser(data?: null): InitialiserEventFilter;
        "OwnershipTransferred(address,address)"(previousOwner?: PromiseOrValue<string> | null, newOwner?: PromiseOrValue<string> | null): OwnershipTransferredEventFilter;
        OwnershipTransferred(previousOwner?: PromiseOrValue<string> | null, newOwner?: PromiseOrValue<string> | null): OwnershipTransferredEventFilter;
        "SafeProxyDeploy(address,uint256)"(user?: PromiseOrValue<string> | null, counter?: null): SafeProxyDeployEventFilter;
        SafeProxyDeploy(user?: PromiseOrValue<string> | null, counter?: null): SafeProxyDeployEventFilter;
        "SafeProxyDeployed(address,address)"(user?: PromiseOrValue<string> | null, proxy?: PromiseOrValue<string> | null): SafeProxyDeployedEventFilter;
        SafeProxyDeployed(user?: PromiseOrValue<string> | null, proxy?: PromiseOrValue<string> | null): SafeProxyDeployedEventFilter;
    };
    estimateGas: {
        deploySafe(owner: PromiseOrValue<string>, nonce: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        deploySafeProxy(counter: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        fallbackModule(overrides?: CallOverrides): Promise<BigNumber>;
        modules(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        modulesLib(overrides?: CallOverrides): Promise<BigNumber>;
        owner(overrides?: CallOverrides): Promise<BigNumber>;
        ownersList(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        renounceOwnership(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        safeProxyFactory(overrides?: CallOverrides): Promise<BigNumber>;
        safeSingleton(overrides?: CallOverrides): Promise<BigNumber>;
        transferOwnership(newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        updateFallbackModule(_fallbackModule: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        updateModulesLib(_modulesLib: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        updateModulesList(_modulesList: PromiseOrValue<string>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        updateSafeProxyFactory(_safeProxyFactory: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        updateSafeSingleton(_safeSingleton: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        userAmounts(arg0: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        users(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
    };
    populateTransaction: {
        deploySafe(owner: PromiseOrValue<string>, nonce: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        deploySafeProxy(counter: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        fallbackModule(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        modules(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        modulesLib(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        owner(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        ownersList(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        renounceOwnership(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        safeProxyFactory(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        safeSingleton(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        transferOwnership(newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        updateFallbackModule(_fallbackModule: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        updateModulesLib(_modulesLib: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        updateModulesList(_modulesList: PromiseOrValue<string>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        updateSafeProxyFactory(_safeProxyFactory: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        updateSafeSingleton(_safeSingleton: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        userAmounts(arg0: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        users(arg0: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
    };
}
